# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from yarn import util, features
parsers = []

def loadParsers():
    parserClasses = util.loadClasses("yarn.formats", Parser)
    for p in parserClasses:
        parsers.append(p)
    parsers.sort(lambda p1, p2: cmp(p2.priority, p1.priority))
    
def parseFile(file, metadata={}):
    """
    try to find a format handler for this file-like object.
    metadata is a dictonary that contains useful information about
    the file.  This dictionary should (but isn't required to) 
    contain at least name and mime-type.
    """
    if not parsers: loadParsers()
    for parser in parsers:
        if parser.canParse(file, metadata):
            return parser(file, metadata)            
    raise Exception, "Unparseable file"
            
def writeToFile(messages, file, format, metadata={}):
    p = getModule(format).Parser()
    p.writeToFile(messages, file, metadata)
    
class Parser(object):
    priority = 0
    
    def __init__(self, stream, metadata):
        self.stream = stream
        self.metadata = metadata

    def canParse(file, metadata={}):
        """
        If this object is able to parse the file, return 1
         otherwise return 0.
        """
        return False
    canParse = staticmethod(canParse)

    def getName(self, path):
        if not path:
            return self.metadata.get('url', '')
        else:
            return path

    def getFeatures(self, path):
        return []
    
    def listMessages(self, path):
        return []
        
    def getMessage(self, key):
        raise NotImplementedError
        
    def getHeaders(self, key):
        raise NotImplementedError
        
    def listPaths(self, path):
        return []

    def getLinks(self, path):
        return []
        
    def getProperties(self, path=None):
        return {'name': ''}
        
    def getUrl(self, path):
        raise NotImplementedError, "Not implemented: getUrl '%s'" % path
    
    def hasLocalPath(self, path):
        return 0
    

class Formatter(object):
    
    def writeToFile(self, messages, file, metadata={}):
        """
        Format messages and write them to the file object provided.
        Metadata may be used to fill in headers as needed.
        """
        raise "not implemented"
