# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from yarn.message import Message
from yarn import formats, url, features
from sgmllib import SGMLParser
import re
from yarn.url import Url
from urlparse import urljoin

class HTMLParser(formats.Parser):
    priority = 1
    
    def canParse(file, metadata):
        if metadata.get('mime-type', '') in ['text/html', '']:
            return True
        else:
            return False

    canParse = staticmethod(canParse)

    def __init__(self, stream, metadata):
        self.stream = stream
        self.stream.seek(0)
        self.metadata = metadata
        self.parsedData = {}
        self.data = stream.read()
        self.metadata['base'] = str(metadata.get('url'))
        parser = MyHTMLParser(self.parsedData)
        try:
            parser.feed(self.data)
        finally:
            parser.close()

    name = property(lambda self: self.parsedData.get('title', ''))
        
    def listMessages(self, path):
        return []

    def listPaths(self, path):
        return []

    def getLinks(self, path):
        links = []
        if path <> "": return links
        knownLinks = {
            ('alternate', 'application/rss+xml'): ("RSS News Feed", 0),
            ('alternate', 'application/atom+xml'): ("Atom News Feed", 0),
            ('edituri', 'application/rsd+xml'): ("RSD File", 1)
            }
        for info in self.parsedData['links']:
            rel = info.get('rel', '').lower()
            mimetype = info.get('type', '').lower()
            title = info.get('title', '')
            href = info.get('href', '')
            description, recurse = knownLinks.get((rel, mimetype), ["", 0])
            if description:
                linkUrl = urljoin(self.metadata['base'], href)
                link = url.Link(title or description,
                                linkUrl, "alternate", mimetype)
                if recurse: link.properties['recurseInto'] = True
                links.append(link)
        return links

    def getFeatures(self, path):
        return [features.LINKS]

class MyHTMLParser(SGMLParser):
    # FIXME: keep track of base href
    
    def __init__(self, parsedData):
        self.data = parsedData
        SGMLParser.__init__(self)
    
    def reset(self):
        SGMLParser.reset(self)
        self.currentTag = ""
        self.data['links'] = []

    def unknown_starttag(self, tag, attrs):
        self.currentTag = tag

    def start_link(self, attrs):
        attributes = {}
        for key, val in attrs: attributes[key.lower()] = val
        self.data['links'].append(attributes)
  
    def handle_data(self, data):
       if self.currentTag == 'title':
            self.data['title'] = self.data.get('title','') + data
