# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from yarn import message, formats

class Parser(formats.Parser):

    def canParse(stream, metadata={}):
        stream.seek(0)
        validTypes = ['text/plain', 'message/rfc822']
        if metadata.has_key('mime-type') and metadata['mime-type'] not in validTypes:
            return 0
        # see if this is a mail file.
        firstLines = stream.readlines(10000)
        hasMessages = 0
        messageCount = 0
        for line in firstLines:
            if line.startswith("From:"): hasMessages = 1
            messageCount = messageCount + line.startswith("From ")
        if messageCount > 1 or hasMessages == 0:
            return 0
        else:
            return 1
    canParse = staticmethod(canParse)
        
    def __init__(self, stream, metadata={}):
        self.stream = stream
        
    def getMessages(self):
        self.stream.seek(0)
        return message.Message(self.stream)
        
    def getHeaders(self):
        return {}
        
    def getChildUrls(self):
        return []
        
class Formatter(formats.Formatter):
    
    def writeToFile(self, messages, f, metadata={}):
        for m in messages:
            f.write(m.asEmail(unixFrom=1))
        
