# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from __future__ import generators
from mailbox import PortableUnixMailbox
from yarn import formats
from yarn.message import Message
import os.path

class Parser(formats.Parser):

    def canParse(stream, metadata={}):
        stream.seek(0)
        if stream.readline().startswith("From "):
            return True
    canParse = staticmethod(canParse)
        
    def getMessages(self):
        self.stream.seek(0)
        self.box = PortableUnixMailbox(self.stream, Message)
        while 1:
            m = self.box.next()
            if not m: break
            yield m
        
    def getHeaders(self):
        parent, name = os.path.split(self.metadata.get('filename',''))
        if name == 'mbox':
            name = os.path.split(parent)[1]
        return {'name': name}
        
    def getChildUrls(self):
        chillins = []
        subfolderDir = self.metadata.get('filename') + ".sbd"
        if os.path.isdir(subfolderDir):
            for path in os.listdir(subfolderDir):
                fullPath = os.path.join(subfolderDir, path)
                if os.path.isfile(fullPath) and Parser.canParse(file(fullPath)):
                    chillins.append({'url': fullPath, 'name': path, 'autoSubscribe': True})
        return chillins
        
class Formatter(formats.Formatter):
    
    def writeToFile(self, messages, f, metadata={}):
        for m in messages:
            f.write(m.asEmail(unixFrom=1))

