# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
RSD (Really Simple Discovery) parser for yarn.
"""

from yarn import formats, url
from xml.sax import make_parser, handler

class RSDParser(formats.Parser):
    priority = 10
    
    apiMappers = {
        'metaweblog': lambda api: url.Link(api.name + 'API',
                                           url.Url('metaweblog:' + api.link),
                                           'alternate')
        }

    def canParse(stream, metadata):
        if metadata.get('mimetype', '') == 'application/rsd+xml':
            return True
        if metadata.get('filename', '').endswith('rsd.xml'):
            return True
        if url.Url(metadata.get('url', '')).path.endswith('rsd.xml'):
            return True
        if metadata.get('mimetype', '') in ('text/xml', 'text/html'):
            # sniff it
            start = stream.read(100)
            if start.count("<rsd "): return True
        return False
    
    canParse = staticmethod(canParse)

    def listMessages(self, path):
        return []

    def listPaths(self, path):
        return []

    def getLinks(self, path):
        if path != "": raise KeyError, "No such path."
        urlInfo = []
        parser = make_parser()
        handler = RSDSaxContentHandler()
        parser.setContentHandler(handler)
        self.stream.seek(0)
        parser.parse(self.stream)
        for service in handler.services:
            for API in service.APIs:
                func = self.apiMappers.get(API.name.strip().lower())
                if func:
                    urlInfo.append(func(API)) # FIXME: figure out what to return
        return urlInfo

# simple classes for storing RSD info
class RSDService:
    def __init__(self):
        self.engineName = ""
        self.APIs = []
        
class RSDAPI:
    def __init__(self):
        self.name = ""
        self.preferred = False
        self.link = ""
        self.blogID = ""
        self.settings = {}

class RSDSaxContentHandler(handler.ContentHandler):

    def __init__(self):
        self.services = []
        self.currentElement = ""
        handler.ContentHandler.__init__(self)
        
    def startElement(self, element, args):
        self.currentElement = element
        if element == 'service':
            self.services.append(RSDService())
        elif element == 'api':
            api = RSDAPI()
            api.name = args.get('name')
            if args.get('preferred', 'false').lower() == 'true':
                api.preferred = True
            else:
                api.preferred = False
            api.blogID = args.get('blogID')
            api.link = args.get('apiLink')
            self.services[-1].APIs.append(api)
        elif element == 'setting':
            pass # maybe handle this someday.

    def characters(self, data):
        if self.currentElement == 'engineName':
            self.services[-1].engineName += data

    def endElement(self, element):
        pass
