# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from extlib import feedparser
import md5
import base64
import rfc822
import sgmllib
import urlparse
from twisted.internet import reactor, threads
from twisted.protocols import http
import re
from yarn import message, formats, features

try:
    from time import strptime
except:
    from yarn.extlib.strptime import strptime
from yarn.extlib.stripogram import html2text

def safeChars(inputstr):
    import re
    return re.sub(r"[^a-zA-Z0-9@\-]", "_", inputstr)

class RSSParser(formats.Parser):
    priority = 100
        
    def canParse(stream, metadata):
        stream.seek(0)
        
        # text/html and text/plain are here to support stupid web servers... grr.
        positiveMimeTypes = [
            'application/rss+xml',
            'application/atom+xml',
        ]            
        validMimeTypes = [
            'application/rdf+xml',
            'application/xml', 
            'text/xml', 
            'text/rdf',
            'text/html', 
            'text/plain',
            '']

        mimeType = metadata.get('mimetype', '').split(';')[0]
        if mimeType in positiveMimeTypes:
            return 1
        elif mimeType not in validMimeTypes:
            return 0
        
        rssDocument = stream.read(1000)
        if re.search("<html", rssDocument, re.IGNORECASE):
            return 0
        if re.search("<(rss|rdf|feed)",
                     rssDocument,
                     re.IGNORECASE+re.DOTALL):
            return 1
        
        return 0
        
    canParse = staticmethod(canParse)
    
    def __init__(self, rssFile, metadata):
        self.metadata = metadata
        self.name = ""
        self.messages = {}
        rssFile.seek(0)
        result = feedparser.parse(rssFile)
        channel = result['channel']
        items = result['items']
        encoding = result.get('encoding', 'utf-8')
        
        if result.get('bozo'): 
            print "worked around invalid XML at %s (%s)" % (
                metadata['url'], result.get('bozo_exception'))
        
        if channel.has_key('title'):
            self.name = channel.get('title', '').decode(encoding)

        for item in items:
            m = message.Message(mimetype='text/html')
            
            m['Subject'] = item.get('title', '').decode(encoding)
            
            if item.has_key('content_encoded'):
                html = item['content_encoded']
            elif item.has_key('description'):
                html = item['description']
            else:
                html = ""

            if html:
                html = html.decode(encoding)
                m.contents = ("<html><body>%s</body></html>" % html)

            if item.has_key('link'):
                m['X-Yarn-Link'] = item['link']
            elif item.has_key('guid'):
                if parser.guidislink:
                    m['X-Yarn-Link'] = item['guid']
            
            if item.has_key('guid'):
                m['Message-ID'] = "<rss:%s>" % item['guid']

            if item.has_key('date'):
                try: m.setTimestamp(item['date_parsed'])
                except: pass
            elif channel.has_key('date'):
                # fall back to the channel's modify date
                try: m.setTimestamp(channel['date'])
                except: pass

            itemAuthorName = message.NameAndAddress()
            itemAuthorName = item.get('author') or item.get('dc_contributor')
            if not itemAuthorName: itemAuthorName = channel.get('title', '')
            fromAddr = message.NameAndAddress()
            fromAddr.name = itemAuthorName.decode(encoding)
            m['From'] = fromAddr
            
            self.messages[m.getMessageId()] = m
        
    def listMessages(self, path):
        return self.messages.keys()
        
    def getMessage(self, path, key):
        return self.messages[key]

    def getName(self, path):
        return self.name

    def getFeatures(self, path):
        return [features.MESSAGES]
