# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import os
from yarn.util import loadClasses
from twisted.internet import reactor, defer
from twisted.python import failure

drivers = []

def loadDrivers():
    from yarn import proxy, url
    for c in loadClasses("yarn.protocols", proxy.Proxy):
        drivers.append(c)
        for protocol, port in c.ports.items():
            url.PORTS[protocol] = port

def openProxy(connector, url, credentials=None):
    if not drivers: loadDrivers()
    for d in drivers:
        if url.protocol in d.protocols:
            proxy = d(url, connector, credentials)
            return proxy.connect().addCallback(lambda _: proxy)

    # no driver found.
    d = defer.Deferred()
    f = failure.Failure(KeyError("No driver found for protocol %s" % url.protocol))
    reactor.callLater(0, d.errback, f)
    return d
