# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from twisted.web.client import HTTPClientFactory
from cStringIO import StringIO
from yarn import proxy, errors, auth
from twisted.internet import reactor, defer

class HTTPProxy(proxy.SimpleProxy):
    protocols = ['http', 'https']
    ports = {'http':80, 'https':443}
    
    def getFile(self):
        if not self.connector.offline:
            return getHeadersAndData(
                self.url, self.credentials).addCallback(self._gotPage)
        else:
            return defer.execute(self.getPageFromCache)
            
    def getPageFromCache(self):
        return StringIO(self.cache.get('data', '')), {
            'url': self.url}       
                
    def _gotPage(self, headersAndData):
        response, headers, data = headersAndData
        headers = headers or {}
        data = data or ""
        cleanHeaders = {}
        for header, val in headers.items():
            cleanHeaders[header.lower()] = val[0]
        return StringIO(data), {
            'url': self.url,
            'mimetype': cleanHeaders.get('content-type', '')}

import base64
def getAuthHeader(credentials):
    user = credentials.get('username', '')
    password = credentials.get('password', '')
    encoded = base64.encodestring("%s:%s" % (user, password)).strip()
    return "Authorization", "Basic " + encoded

def getHeadersAndData(url, credentials, contextFactory=None, headers={}):
    creds = url.getCredentials() or credentials
    if creds:
        header, value = getAuthHeader(creds)
        headers[header] = value
    if url.path == '': url.path = '/'
    factory = HTTPClientFactory(str(url), headers=headers)
    factory.status = None
    #print "OPENING ", url.server, "port", url.port, "with", url.protocol
    if url.protocol == 'https':
        from twisted.internet import ssl
        if contextFactory is None:
            contextFactory = ssl.ClientContextFactory()
        reactor.connectSSL(url.server, url.port, factory, contextFactory)
    else:
        reactor.connectTCP(url.server, url.port, factory)
    factory.deferred.addCallback(
        lambda data, fact: (fact.status, fact.response_headers, data),
        factory)
    factory.deferred.addErrback(handleHttpError, factory)
    return factory.deferred

def handleHttpError(error, factory):
    #print "HTTP: ERROR, status", factory.status
    if factory.status == '401':
        raise errors.CredentialsRequired(auth.UserPassCredentials,
                                         factory.message)
    elif factory.status == '403':
        raise errors.LoginFailed(auth.UserPassCredentials, factory.message)
    else:
        return error

