# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from twisted.protocols import jabber, xmlstream
from twisted.protocols.jabber import client, jid
from twisted.internet import defer, reactor
from twisted.xish import domish
from yarn import message, proxy

PROTOCOLS = ['jabber']
PORTS = {'jabber': 5222}

def openProxy(url, connector):
    print 'opening proxy'
    p = JabberProxy(url, connector)
    d = p.connect()
    d.addCallback(lambda _: p)
    return d

class JabberProxy(proxy.Proxy):
    
    def __init__(self, url, connector):
        proxy.Proxy.__init__(self, url, connector)
        self.stream = None
        self.connecting = None
        
    def getJID(self):
        return '%s@%s%s' % (self.url.username,
                            self.url.server,
                            self.url.path or '/Hep')

    def connect(self):
        print 'connecting...'
        myJid = jid.JID(self.getJID())
        print myJid
        self.factory = client.basicClientFactory(myJid, self.url.password)
        self.factory.addBootstrap(xmlstream.STREAM_AUTHD_EVENT, self.authd)
        print self.factory
        self.connecting = defer.Deferred()
        reactor.connectTCP(self.url.server, int(self.url.port), self.factory)
        return self.connecting
        
    def authd(self, stream):
        print "authorization successful."
        self.stream = stream
        # send presence indicator
        presence = domish.Element(("jabber:client","presence"))
        stream.send(presence)
        stream.addObserver("/message", self.gotMessage)
        if self.connecting:
            self.connecting.callback(1)
            self.connecting = None
        
    def listMessages(self, path=None):
        return defer.succeed([])
        
    def getProperties(self, path=None):
        return defer.succeed({'name': str(self.url)})
        
    def postMessage(self, message, jid):
        print "posting message"
        msgXml = domish.Element(("", "message"))
        msgXml['to'] = jid
        msgXml['from'] = self.getJID()
        msgXml.addUniqueId()
        msgXml.addElement(('','body'), content=message.getText())
        if message['title']:
            msgXml['subject'] = message['title']
            msgXml['type'] = 'normal'
        else:
            msgXml['type'] = 'chat'
        print msgXml.toXml()
        self.stream.send(msgXml)
        
    def gotMessage(self, msgXml):
        print "Got message!"
        print msgXml.toXml()
        body = msgXml.body
        if body:
            m = message.Message()
            m['to'] = msgXml.getAttribute('to') or ''
            m['from'] = msgXml.getAttribute('from') or ''
            m['subject'] = str(msgXml.subject)
            m.setText(str(body))
            if msgXml.html:
                m.setHTML(msgXml.html.toXml())
            #self.handleEvent(messaging.EVENT_NEW, m)
        
