# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
server support for test cases.
"""
from twisted.internet import reactor
from twisted.web import server, static, resource
from urlparse import urljoin
import os

class WebServer:
    """
    Web server for testing - listens on a free port, serves contents
    of yarn/tests/files directory.
    """
    def __init__(self):
        self.port = None

    def start(self):
        pageDir = os.path.join(os.path.split(__file__)[0], 'files')
        baseResource = static.File(pageDir)
        baseResource.putChild('auth', HttpAuthPage())
        self.site = server.Site(baseResource)
        self.port = reactor.listenTCP(0, self.site) # listen on a free port
        reactor.iterate(); reactor.iterate()

    def stop(self):
        self.port.stopListening()
        reactor.iterate(); reactor.iterate()

    def getUrl(self, path=''):
        hostInfo = self.port.getHost()
        baseUrl = "http://localhost:%i/" % (hostInfo.port)
        return urljoin(baseUrl, path)

class HttpAuthPage(resource.Resource):

    content = "Hello World"
    username = 'admin'
    password = 'pass'

    def render(self, request):
        user = request.getUser()
        password = request.getPassword()
        if (user, password) != (self.username, self.password):
            request.setResponseCode(401) # auth required
            request.setHeader('WWW-Authenticate', 'Basic realm=localhost')
            return "Access denied!"
        else:
            return self.content

if __name__ == '__main__':
    w = WebServer()
    w.start()
    print 'serving pages at', w.getUrl()
    reactor.run()
