# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from twisted.trial import unittest
from yarn import auth

class CredTester(unittest.TestCase):
    def testCreateCreds(self):
        up = auth.UserPassCredentials('abe', 'test')
        self.assertEquals(up, {'username':'abe', 'password': 'test'})

    def testCreateCreds2(self):
        up2 = auth.UserPassCredentials('abe') # OK
        self.assertEquals(up2, {'username':'abe'})

    def testCreateCreds3(self):
        self.assertRaises(TypeError,
                          auth.UserPassCredentials,
                          'abe', 'blah', 'blah')

    def testNullCredentials(self):
        nullCreds = auth.UserPassCredentials("","")
        self.assertEquals(nullCreds, auth.AnonymousCredentials())
        self.assertEquals(nullCreds.getDigest(), "")

from cStringIO import StringIO
class KeychainTester(unittest.TestCase):
    def setUp(self):
        self.chain = auth.Keychain()
        userpass = auth.UserPassCredentials('user', 'pass')
        token = auth.TokenCredentials('blah')
        self.chain['http://www.mywebsite.com/'] = userpass
        self.chain['whatever://stuff'] = token
    
    def testXML(self):
        "test XML round-tripping"
        output = StringIO()
        output.write(self.chain.getXml())
        chain2 = auth.Keychain()
        output.seek(0)
        chain2.loadXml(output)
        self.assertEquals(self.chain, chain2)

    def testUrlMatches(self):
        creds1 = auth.UserPassCredentials('user1', 'pass1')
        creds2 = auth.UserPassCredentials('user2', 'pass2')
        k = auth.Keychain()
        k['http://www.site.com/'] = creds1
        k['http://www.site.com/special/'] = creds2

        self.assertEquals(k.get('http://www.site.com/page1'), creds1)
        self.assertEquals(k.get('http://www.site.com/special/'), creds2)
        self.assertEquals(k.get('http://www.site.com/special/page'), creds2)
        self.assertEquals(k.get('http://www.site.com/special/stuff#s'), creds2)

