# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from twisted.trial import unittest, util
from yarn.formats import html
from yarn import remote
from server import WebServer
import os.path

class HtmlFormatTester(unittest.TestCase):

    def setUp(self):
        thisdir = os.path.split(__file__)[0]
        htmlfile = os.path.join(thisdir, 'files', 'html', 'rssatomlinks.html')
        self.parser = html.HTMLParser(file(htmlfile),
                      {'url': 'http://www.mysite.com/index.html'})

    def testLinks(self):
        links = self.parser.getLinks('')
        links.sort(lambda link1, link2: cmp(link1.mimetype, link2.mimetype))
        self.assertEquals(len(links), 2)
        self.assertEquals(links[0].url, 'http://www.mysite.com/atom.xml')
        self.assertEquals(links[0].mimetype, 'application/atom+xml')
        self.assertEquals(links[0].title, 'MY WICKED COOL ATOM FEED')
        self.assertEquals(links[1].url, 'http://www.mysite.com/rss.xml')
        self.assertEquals(links[1].mimetype, 'application/rss+xml')
        self.assertEquals(links[1].title, 'RSS News Feed')

class HtmlFolderTester(unittest.TestCase):
    def setUp(self):
        self.webserver = WebServer()
        self.webserver.start()

    def tearDown(self):
        self.webserver.stop()
        del(self.webserver)

    def testRSD(self):
        f = remote.Folder()
        util.deferredResult(
            f.connect(self.webserver.getUrl('/html/rsdlink.html')))
        links = util.deferredResult(f.getAlternateLinks())
        print [l.url for l in links]
        self.assertEquals(len(links), 1)
        self.assertEquals(links[0].url,
                          "metaweblog:http%3A//example.com/xml/rpc/url")

    def testGoodBadLinks(self):
        f = remote.Folder()                      
        util.deferredResult(f.connect(
            self.webserver.getUrl('/html/goodbadlinks.html')))
        links = util.deferredResult(f.getAlternateLinks())
        # the referenced RSD file doesn't actually exist, so it shouldn't
        # be included in the results.
        self.assertEquals(len(links), 1)
        print links
        self.assertEquals(links[0].mimetype, 'application/rss+xml')
        
                      
    
    
