# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from twisted.trial import unittest
from twisted.trial.util import deferredError, deferredResult
from twisted.web import static, server, resource
from twisted.python import failure
from yarn import remote, auth, errors
import os.path
from twisted.internet import reactor
from twisted.internet.error import ConnectionRefusedError
from server import WebServer

class HttpTester(unittest.TestCase):

    def setUp(self):
        self.connector = remote.Connector()
        self.server = WebServer()
        self.server.start()

    def tearDown(self):
        del(self.connector)
        self.server.stop()
        del(self.server)

    def assertViewError(self, url, errorClass, keychain=None):
        f = remote.Folder(keychain=keychain)
        try:
            deferredResult(f.connect(url))
            raise Exception, "connect() succeeded, failure expected!"
        except failure.Failure, e:
            self.assertEquals(e.check(errorClass), errorClass)
            if errorClass == errors.CredentialsRequired:
                self.assertEquals(e.value.credentialsClass,
                                  auth.UserPassCredentials)

    def testConnectFail(self):
        self.assertViewError('http://127.0.0.1:99999',
                             ConnectionRefusedError)
    def testAuthFail(self):
        self.assertViewError(self.server.getUrl('auth'),
                             errors.CredentialsRequired)
                        
    def testAuthSucceed(self):
        u = self.server.getUrl('auth')
        k = auth.Keychain()
        creds = auth.UserPassCredentials('admin', 'pass')
        k[u] = creds
        f = remote.Folder(keychain=k)
        deferredResult(f.connect(u, self.connector))

    
