# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

from yarn import remote, url
from twisted.internet import reactor, defer
from twisted.trial import unittest, util
import server
import tempfile, os

class OPMLTester(unittest.TestCase):
    def setUp(self):
        self.webserver = server.WebServer()
        self.webserver.start()
        self.folder = remote.Folder()
        # copy opml file to temp file for editing.
        opmlFile = file(os.path.join(os.path.split(__file__)[0], 'files',
                                     'opml', 'sites.opml'))
        self.file = tempfile.NamedTemporaryFile(suffix='.opml')
        self.file.write(opmlFile.read())
        self.file.flush()
        opmlFile.close()
        util.deferredResult(self.folder.connect(self.file.name))
            #self.folder.connect(self.webserver.getUrl('/opml/sites.opml')))

    def tearDown(self):
        del(self.folder)
        self.file.close()
        del(self.file)
        self.webserver.stop()
        del(self.webserver)

    def testOpml(self):
        paths = self.folder.subFolders.keys()
        # try adding a path
        newPath = url.Link('Test Site', self.webserver.getUrl('/rss/rss.xml'))
        util.deferredResult(self.folder.createLink(newPath))
        # make sure a new folder was added.
        self.assertEquals(len(self.folder.subFolders), len(paths)+1)
        # now delete everything
        for f in self.folder.subFolders.keys():
            util.deferredResult(self.folder.deleteFolder(f))
        self.assertEquals(len(self.folder.subFolders), 0)
        # create new links, make sure they can be read back unchanged.
        
