# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

import re
import os.path
from urlparse import urlparse, urljoin, urlunparse
import urllib
import auth, protocols

PORTS = {}

class Url(object):
    
    def __init__(self, urlStr):        
        urlStr = str(urlStr)

        self._protocol = ""
        self.username = ""
        self.password = ""
        self.server = ""
        self._port = 0
        self.path = ""
        self.args = ""
        self.fragment = ""

        # fix windows paths, so urlparse doesn't think the drive letter
        # is a protocol
        reWinPath = re.compile(r"^([a-zA-Z]:[\\\/])")
        winPath = reWinPath.match(urlStr)
        if winPath: 
            urlStr = 'file://' + os.path.normpath(urlStr)

        bits = urlparse(urlStr)
        self._protocol = bits[0]
        server = bits[1]
        self.path = bits[2]
        if bits[3]: self.path += ';' + bits[3]
        self.args = bits[4]
        self.fragment = bits[5]
        # fragment is always client-site - it should never be part of the path
        if not self.fragment and self.path.count('#'):
            self.path, self.fragment = self.path.split('#', 1)

        if self._protocol == 'file' and server and not self.path:
             server, self.path = self.path, server

        # try to detect host information in the path
        # this is needed for mailto: urls, or http urls without the http.
        host = re.compile(r'^([A-Za-z0-9\-]*\.)+[A-Za-z]{2,5}$')
        ipAddr = re.compile(r'^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$')
        if not server:
            testStr = self.path.split('@')[-1].split('/')[0]
            if host.match(testStr) or ipAddr.match(testStr) or testStr == 'localhost':
                if self.path.count('/'):
                    server, path = self.path.split('/', 1)
                    self.path = '/' + path
                else:
                    server, self.path = self.path, ''
                #if not self._protocol: self._protocol = 'http' # http default

        # check for authentication info in server
        if server.count('@'):
            userpass, server = server.split('@', 1)
            if userpass.count(':'):
                self.username, self.password = userpass.split(':')
            else:
                self.username = userpass
                self.password = ''
        if server.count(':'):
            server, port = server.split(':')
            try:
                self._port = int(port or 0)
            except:
                pass

        self.path = urllib.unquote_plus(self.path)
        self.fragment = urllib.unquote_plus(self.fragment)
        self.server = server

    def _getPort(self):
        if not PORTS: protocols.loadDrivers() # FIXME - better interface here
        if (not self._port) and PORTS.has_key(self._protocol):
            return PORTS[self._protocol]
        else:
            return self._port
        
    def _setPort(self, port):
        self._port = int(port)
        
    port = property(_getPort, _setPort)

    def _getProtocol(self):
        if not self._protocol and self.server:
            return 'http'
        elif not self._protocol:
            return 'file'
        else:
            return self._protocol

    def _setProtocol(self, protocol):
        self._protocol = protocol

    protocol = property(_getProtocol, _setProtocol)

    def __getattr__(self, key):
        "handle string methods"
        if self.__dict__.has_key(key):
            return self.__dict__[key]
        elif hasattr("", key):
            return getattr(str(self), key)
        else:
            raise AttributeError, ("'Url' object has no attribute '%s'" % key)

    def __str__(self):
        # fix paths for file urls
        if self.username:
            if self.password:
                serverStr = "%s:%s@%s" % (
                    self.username, self.password, self.server)
            else:
                serverStr = "%s@%s" % (self.username, self.server)
        else:
            serverStr = self.server
        if self._port:
            serverStr += ":%i" % (self._port)

        if self.protocol == 'file':
            return 'file://' + self.path
        else:
            return urlunparse([self.protocol, serverStr,
                               urllib.quote(self.path),
                              '', self.args, urllib.quote(self.fragment)])
        
    def __unicode__(self, encoding=None):
        return unicode(self.__str__())


    def __cmp__(self, other): return cmp(str(self), other)
    def __repr__(self): return 'Url("%s")' % str(self)
    def __add__(self, other): return str(self) + other

    def getCredentials(self):
        return auth.UserPassCredentials(self.username, self.password)

    def getUrl(self, urlfragment):
        "return a fully-qualified URL based  on the url or url fragment given"
        result = Url(urljoin(str(self), str(urlfragment)))
        return result

class Link(object):
    "represents a link to a Url, with name and metadata"
    
    def __init__(self, title, url, rel='child', mimetype=''):
        self.title = title
        self.url = Url(url)
        self.rel = rel
        self.mimetype = mimetype
        self.properties = {}
