# This file is part of yarn, a library for working with messaging systems.
# Copyright (C) 2004 Abe Fettig
#
# yarn is free software; you can redistribute it and/or
# modify it under the terms of version 2 of the GNU General
# Public License as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

"""
yarn.util - utility functions for yarn
"""
import glob, os, os.path

def loadModules(packageName):
    """
    load and return all the modules in the path dir, which must be
    available to Python as package packageName.
     """
    mods = []
    if packageName.count('.'):
        packageBits = packageName.split(".")
        parentPackage = ".".join(packageBits[0:-1])
    else:
        parentPackage = ""
    # correct usage is __import__('package.subpackage', {}, {}, 'package')
    packageMod = __import__(packageName, {}, {}, parentPackage)
    dir, file = os.path.split(packageMod.__file__)
    for modFile in glob.glob1(dir, "*.py"):
        modName, ext = os.path.splitext(os.path.basename(modFile))
        if modName == "__init__": continue
        fullName = ".".join([packageName, modName])
        try:
            mod = __import__(fullName, {}, {}, packageName)
            mods.append(mod)
        except ImportError, e:
            print "Unable to load module: %s" % modFile, e
    return mods

def loadClasses(packageName, baseClass):
    """
    return all classes of type baseClass, searching all modules
    in the directory dir, which is available as package packageName
    """
    classes = []
    mods = loadModules(packageName)
    for mod in mods:
        for itemName in dir(mod):
            item = getattr(mod, itemName)
            try:
                if issubclass(item, baseClass):
                    classes.append(item)
            except TypeError:
                pass # if item isn't a class, ignore the error
    return classes
